.TH DALIB 3 "November, 1996" DALIB "Digital Audio Tape support Library"
.SH NAME
da_open \- Open a file or device to be used as DAT device.
.SH SYNOPSIS
.B #include <sys/fcntl.h>
.B #include "dataudiolib.h"
.sp
.B int
.br
.BI "da_open( char *" name " , int mode)"
.SH DESCRIPTION
The
.B da_open()
function opens a filename and initialises internal status for
the resulting descriptor in the library.
If 
.B fd
is associated with a tape device, this functions tries to load the last
saved state of this tape device (if present).
.br
You have to get a descriptor using either
.B da_init
or 
.B da_open
before you can use the other functions.
.SH PARAMETERS
.SS Input Parameters
.IP name
A filename in the UNIX filesystem.
.IP mode
May contain the same flags that can be passed to 
.B open(3)
as defined in <sys/fcntl.h>. The library uses O_RDONLY,O_WRONLY,
O_RDWR and O_NDELAY. (The mode is also passed to the open(3) function
when opening the device.)
.SH RETURN VALUE
This function returns an integer descriptor to be used with other da_xxx
functions.
.br
When an error condition arises, this routine returns -1 and sets errno to
indicate the reason for the error.  Possible values of errno are:
.TP
EINVAL
The filedescriptor is invalid.
.TP
other
The usual errors returned by 
.B open(3)
and
.B fstat(3)
\&.
.SH USAGE EXAMPLE
.PP
Open $TAPE as a DAT device.
.PP
.nf
		char *tape;

		tape=getenv("TAPE");
		if (tape==NULL)
			tape="/dev/rmt/0cn"; /* reasonable default */
		datfd=da_open(tape,O_RDWR);
.fi
.SH SEE ALSO
.B da_intro(3),da_open(3)
.SH AUTHOR
Marcus Meissner <msmeissn@cip.informatik.uni-erlangen.de>
